{% if import_record.fallback -%}
{% if import_record.min_version -%}
if sys.version_info >= {{ import_record.min_version }}:
    {% include "common/import_record_string.py.jinja2" with context -%}{{ "\n" -}}
else:
    {% with import_record = import_record.fallback -%}
    {% include "common/import_record_string.py.jinja2" with context -%}
    {% endwith -%}
{% else -%}
try:
    {% include "common/import_record_string.py.jinja2" with context -%}{{ "\n" -}}
except ImportError:
    {% with import_record = import_record.fallback -%}
    {% include "common/import_record_string.py.jinja2" with context -%}
    {% endwith -%}
{% endif -%}
{% else -%}
{% include "common/import_record_string.py.jinja2" with context -%}
{% endif -%}
