{{ type_def.name -}}
{{ ' = TypedDict("' -}}
{{ type_def.name -}}
{{ '", {' -}}
{% for child in type_def.iterate_children() -%}
    {{ child.render(type_def.name) -}}
    {{ ", " -}}
{% endfor -%}
{{ "}" -}}
{{ ")\n" -}}
