
{% if package.service_resource.sub_resources %}
### Resources

Type annotations and code completion for additional resources
from `#!python session.resource("{{ package.service_name.boto3_name }}").*`.

```python
# {{ package.service_resource.sub_resources[0].name }} usage example

from {{ package.name }}.service_resource import {{ package.service_resource.sub_resources[0].name }}

def get_resource() -> {{ package.service_resource.sub_resources[0].name }}:
    return resource.{{ package.service_resource.sub_resources[0].name }}(...)
```

{% for sub_resource in package.service_resource.sub_resources -%}
- [{{ sub_resource.name }}]({{ service_name.get_md_doc_link('service_resource', sub_resource.name) }}){{ '\n' -}}
{% endfor %}
{% endif %}
