"""
Setup script for {{ package.pypi_name }}.
"""
from pathlib import Path

from setuptools import setup


LONG_DESCRIPTION = (Path(__file__).parent / "README.md").read_text()


setup(
    name="{{ package.pypi_name }}",
    version="{{ package.version }}",
    packages=[
        "{{ package.name }}",
    ],
    url="{{ builder_repo_url }}",
    license="MIT License",
    author="Vlad Emelianov",
    author_email="vlad.emelianov.nz@gmail.com",
    description="Type annotations for {{ package.library_name }} {{ package.library_version }} master module generated with mypy-boto3-builder {{ builder_version }}",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Environment :: Console",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: CPython",
        "Typing :: Typed",
    ],
    keywords='boto3 type-annotations boto3-stubs mypy mypy-stubs typeshed autocomplete auto-generated',
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    python_requires='>=3.7',
    project_urls={
        'Documentation': '{{ package.get_local_doc_link() }}',
        'Source': '{{ builder_repo_url }}',
        'Tracker': '{{ builder_repo_url }}/issues',
    },
    install_requires=[
        "boto3",
        "typing-extensions>=4.1.0; python_version<\"3.12\"",
    ],
    package_data={"{{ package.name }}": ["py.typed"]},
    zip_safe=False,
)
