{% if package.type_defs %}
### Type definitions

`{{ package.name }}.type_defs` module contains structures and shapes assembled
to typed dictionaries and unions for additional type checking.

Full list of `{{ service_name.class_name }}` TypeDefs can be found in [docs]({{ package.get_doc_link('type_defs') }}).

```python
from {{ package.name }}.type_defs import {{ package.type_defs[0].name }}

def get_value() -> {{ package.type_defs[0].name }}:
    return {
      ...
    }
```

{% endif -%}
